from typing import Literal

from pydantic import BaseModel


class UserStatistics(BaseModel):
    userId: int
    totalOrdersCount: int
    successRate: str
    successPercent: int


class User(BaseModel):
    userId: int
    nickname: str
    avatarCode: str
    statistics: UserStatistics
    isVerified: bool
    lastOnlineMinutesAgo: int
    onlineStatus: str


class AvailableVolume(BaseModel):
    currencyCode: str
    amount: str


class Price(BaseModel):
    type: str
    baseCurrencyCode: str
    quoteCurrencyCode: str
    value: str
    estimated: str


class OrderAmountLimits(BaseModel):
    currencyCode: str
    min: str
    max: str
    approximate: bool


class OrderVolumeLimits(BaseModel):
    currencyCode: str
    min: str
    max: str
    approximate: bool


class ChangeLogItem(BaseModel):
    status: str
    createDateTime: str
    initiatorUserId: int


class ChangeLog(BaseModel):
    items: list[ChangeLogItem]


class TakerFilter(BaseModel):
    accountAge: str
    completedOrders: str
    userRating: str


class FeeAvailableVolume(BaseModel):
    currencyCode: str
    amount: str


class Fee(BaseModel):
    rate: str
    availableVolume: FeeAvailableVolume


class KeyVal(BaseModel):
    name: Literal["PAYMENT_DETAILS_NUMBER", "PHONE"] = "PAYMENT_DETAILS_NUMBER"
    value: str


class BanksIn(BaseModel):
    name: Literal["BANKS"] = "BANKS"
    value: list[str]


class Bank(BaseModel):
    code: str
    nameRu: str
    nameEn: str


class Banks(BanksIn):
    value: list[Bank]


class Attrs(BaseModel):
    version: Literal["V1"] = "V1"
    values: list[KeyVal]


class AttrsV2In(BaseModel):
    version: Literal["V2"]
    values: list[KeyVal | BanksIn]


class AttrsV2(AttrsV2In):
    values: list[KeyVal | Banks]


class FiatPydIn(BaseModel):
    paymentMethodCode: str
    currencyCode: str
    name: str
    attributes: Attrs | AttrsV2In


class PayMeth(BaseModel):
    code: str
    name: str
    originNameLocale: str
    nameEng: str


class FiatPyd(BaseModel):
    id: int
    userId: int
    paymentMethod: PayMeth
    currency: str
    name: str = ""
    attributes: Attrs | AttrsV2


class AdPyd(BaseModel):
    id: int
    number: str
    user: User
    type: str
    availableVolume: AvailableVolume
    paymentConfirmTimeout: str
    price: Price
    orderAmountLimits: OrderAmountLimits
    orderVolumeLimits: OrderVolumeLimits
    status: str
    createDateTime: str
    comment: str
    changeLog: ChangeLog
    takerFilter: TakerFilter
    orderConfirmationTimeout: str
    orderAcceptTimeout: str
    paymentDetails: list[FiatPyd]
    fee: Fee
